part of json_mapper_fixnum.test;

@jsonSerializable
class Int32IntData {
  Int32 int32;

  Int32IntData(this.int32);
}

@jsonSerializable
class Int64IntData {
  Int64 int64;

  Int64IntData(this.int64);
}

testBasics() {
  test("Int32 converter", () {
    // given
    final String rawString = "1234567890";
    final String json = '{"int32":"${rawString}"}';

    // when
    String targetJson =
        JsonMapper.serialize(Int32IntData(Int32.parseInt(rawString)), '');
    // then
    expect(targetJson, json);

    // when
    Int32IntData target = JsonMapper.deserialize(json);
    // then
    expect(rawString, target.int32.toString());
  });

  test("Int64 converter", () {
    // given
    final String rawString = "1234567890123456789";
    final String json = '{"int64":"${rawString}"}';

    // when
    String targetJson =
        JsonMapper.serialize(Int64IntData(Int64.parseInt(rawString)), '');
    // then
    expect(targetJson, json);

    // when
    Int64IntData target = JsonMapper.deserialize(json);
    // then
    expect(rawString, target.int64.toString());
  });

  test("List<Int32>", () {
    // given
    final String json = '["2112454933","2112454934"]';

    // when
    List<Int32> instance = JsonMapper.deserialize(json);
    // then
    expect(instance, TypeMatcher<List<Int32>>());
    expect(instance[0], Int32(2112454933));
    expect(instance[1], Int32(2112454934));

    // when
    final targetJson = JsonMapper.serialize(instance, '');
    // then
    expect(targetJson, json);
  });

  test("Set<Int32>", () {
    // given
    final String json = '["2112454933","2112454934"]';

    // when
    Set<Int32> instance = JsonMapper.deserialize(json);
    // then
    expect(instance, TypeMatcher<Set<Int32>>());
    expect(instance.elementAt(0), Int32(2112454933));
    expect(instance.elementAt(1), Int32(2112454934));

    // when
    final targetJson = JsonMapper.serialize(instance, '');
    // then
    expect(targetJson, json);
  });

  test("List<Int64>", () {
    // given
    final String json = '["1234567890123456789","1234567890123456787"]';

    // when
    List<Int64> instance = JsonMapper.deserialize(json);
    // then
    expect(instance, TypeMatcher<List<Int64>>());
    expect(instance[0], Int64(1234567890123456789));
    expect(instance[1], Int64(1234567890123456787));

    // when
    final targetJson = JsonMapper.serialize(instance, '');
    // then
    expect(targetJson, json);
  });

  test("Set<Int64>", () {
    // given
    final String json = '["1234567890123456789","1234567890123456787"]';

    // when
    Set<Int64> instance = JsonMapper.deserialize(json);
    // then
    expect(instance, TypeMatcher<Set<Int64>>());
    expect(instance.elementAt(0), Int64(1234567890123456789));
    expect(instance.elementAt(1), Int64(1234567890123456787));

    // when
    final targetJson = JsonMapper.serialize(instance, '');
    // then
    expect(targetJson, json);
  });
}
